CBH = {}

--Default setting for addon. When new user defined values are saved, these values are ignored by the game
CBH.Default = {
	["notificationOption"] = 1,
	["autoTeleportation"] = true,
	["wasInHouse"] = false,
	["includePreviewHouses"] = true,
	["includeOtherPlayersHouses"] = true,
	["playerIsAGuest"] = false,
	["House"] =
        {
			[1] = "ID",
			[2] = "Name",
			[3] = "Owner",
			["ID"] = 0,
			["Owner"] = "",
			["Name"] = "",
		},
}

CBH.name = "ComingBackHome"
CBH.version = "1.5"

local TLW = nil -- TopLevelWindow for notifications
local WM = GetWindowManager() --reference to game's WindowManager

function CBH:Initialize()

	--slash command and function which is called with this command
	SLASH_COMMANDS["/cbh"] = CBH.TeleportNow

	--getting variables from saved settings (or set default values)
	self.savedVariables = ZO_SavedVars:New("ComingBackHomeSavedVariables", 1.4, nil, CBH.Default)
	--self.savedVariables = ZO_SavedVars:NewAccountWide("ComingBackHomeSavedVariables", 1.5, nil, CBH.Default)

	if self.savedVariables.notificationOption == 1 then
		CBH:CreateControl() --notification label control
	end

	CBH:CreateSettingsMenu() --in ComingBackHomeMenu.lua file (needs LibAddonMenu-2.0)

	--PlayerActivated event occurs after every loading screen, addon will check current zone then
	EVENT_MANAGER:RegisterForEvent(CBH.name, EVENT_PLAYER_ACTIVATED, CBH.Player_Activated)
	--InstanceKickTimeUpdate occurs when player left a group in dungeon/trial, addon will teleport the player to house then
	EVENT_MANAGER:RegisterForEvent(CBH.name, EVENT_INSTANCE_KICK_TIME_UPDATE, CBH.OnKickTimerUpdate)
	--////Doesn't work in PvP areas
	--BattlegroundStateChanged returns "PostGame" when match is over, addon will teleport the player to house then
	--EVENT_MANAGER:RegisterForEvent(CBH.name, EVENT_BATTLEGROUND_STATE_CHANGED, CBH.OnBGStateChanged)
	--////
end

function CBH.Player_Activated(...)

	if not IsUnitInDungeon("player") then
		CBH:UpdateCurrentHouseInfo()
	end
end

function CBH.UpdateCurrentHouseInfo()

	if GetCurrentZoneHouseId() ~= 0 then
		CBH.savedVariables.wasInHouse = true
		local currentHouseID = GetCurrentZoneHouseId()
		local currentOwner = GetCurrentHouseOwner()
		if IsOwnerOfCurrentHouse() then --if it's player's house then save it
			CBH.savedVariables.playerIsAGuest = false
			CBH:SaveHouseInfo(currentHouseID, GetUnitDisplayName("player"))
		elseif currentOwner == "" then
			if CBH.savedVariables.includePreviewHouses then
				CBH.savedVariables.playerIsAGuest = false
				CBH:SaveHouseInfo(currentHouseID, "Preview")
			end
		else
			if CBH.savedVariables.includeOtherPlayersHouses then
				CBH.savedVariables.playerIsAGuest = true
				CBH:SaveHouseInfo(currentHouseID, currentOwner)
			end
		end
	else
		CBH.savedVariables.wasInHouse = false
	end
end

function CBH:SaveHouseInfo(_houseID, _ownerName)

	CBH.savedVariables.House.ID = _houseID
	if _houseID ~= 0 then
		local collectibleID = GetCollectibleIdForHouse(_houseID)
		CBH.savedVariables.House.Name = GetCollectibleName(collectibleID)
	end
	CBH.savedVariables.House.Owner = _ownerName
end

function CBH.OnKickTimerUpdate(eventCode, timeRemainingMs)

	--jumping back to last house when not in a group anymore
	if CBH.savedVariables.House.ID ~= 0 and CBH.savedVariables.wasInHouse == true and CBH.savedVariables.autoTeleportation == true then
		CBH:StartTeleportation()
	end
end

--[[////Doesn't work, because jumping from PvP areas is forbidden////
function CBH.OnBGStateChanged(eventCode, previousState, currentState)
	if currentState == BATTLEGROUND_STATE_POSTGAME and CBH.savedHouseID ~= 0 and CBH.savedHouseID ~= nil then
		CBH:StartTeleportation()
	end
end
]]--

function CBH.StartTeleportation()

	if not IsUnitInCombat("player") then
		if CBH.savedVariables.notificationOption ~= 3 then
			CBH:Countdown()
		end
		--run function after countdown, and check if still not in combat
		zo_callLater(function()
			if not IsUnitInCombat("player") then
				StopAllMovement()
				CBH:TeleportNow()
			else
				CBH:UnitInCombatNotification()
				zo_callLater(function() CBH.StartTeleportation() end, 5000)
			end
		end, 5000)
	else
		CBH:UnitInCombatNotification()
		zo_callLater(function() CBH.StartTeleportation() end, 5000)
	end
end

--countdown is over, let's start
function CBH.TeleportNow()
	if CBH.savedVariables.House.ID ~= 0 then
		--ESO has two options:
		--for player's house (or preview)...
		if CBH.savedVariables.House.Owner == GetUnitDisplayName("player")
			or CBH.savedVariables.House.Owner == "Preview" then
				RequestJumpToHouse(CBH.savedVariables.House.ID)
		--...and some specific player's house
		else
			JumpToSpecificHouse(CBH.savedVariables.House.Owner, CBH.savedVariables.House.ID)
		end
		if CBH.savedVariables.notificationOption == 1 then
			TLW:SetHidden(false) --make 'TopLevelWindow' (a parent of our label control) visible
			CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner))
			zo_callLater(function() TLW:SetHidden(true) end, 3000) --hide label in the end of countdown
		elseif CBH.savedVariables.notificationOption == 2 then
			d(zo_strformat("CBH: ", GetString(COMINGBACKHOME_UI_COMING_BACK_TO), " ",  CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner))
		end
	else
		--print in chat that there's no last house data saved
		d(GetString(COMINGBACKHOME_NO_HOUSE_TO_TELEPORT))
	end
end

function CBH.Countdown()

	if CBH.savedVariables.notificationOption == 1 then
		TLW:SetHidden(false) --make 'TopLevelWindow' (a parent of our label control) visible
		CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, "\n", 5))
		zo_callLater(function() CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, "\n", 4)) end, 1000)
		zo_callLater(function() CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, "\n", 3)) end, 2000)
		zo_callLater(function() CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, "\n", 2)) end, 3000)
		zo_callLater(function() CBH_Label:SetText(zo_strformat(GetString(COMINGBACKHOME_UI_COMING_BACK_TO), "\n", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, "\n", 1)) end, 4000)
		zo_callLater(function() TLW:SetHidden(true) end, 5500) --hide label in the end of countdown
	elseif CBH.savedVariables.notificationOption == 2 then
		d(zo_strformat("CBH: ", GetString(COMINGBACKHOME_UI_COMING_BACK_TO), " ",  CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, " ", 3))
		zo_callLater(function() d(zo_strformat("CBH: ", GetString(COMINGBACKHOME_UI_COMING_BACK_TO), " ", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, " ", 2)) end, 1000)
		zo_callLater(function() d(zo_strformat("CBH: ", GetString(COMINGBACKHOME_UI_COMING_BACK_TO), " ", CBH.savedVariables.House.Name, CBH.savedVariables.House.Owner, " ", 1)) end, 3000)
	end
end

function CBH.UnitInCombatNotification()
	if CBH.savedVariables.notificationOption == 1 then
		TLW:SetHidden(false)
		CBH_Label:SetText(GetString(COMINGBACKHOME_UI_WAITING_FOR_COMBAT_END))
		zo_callLater(function() TLW:SetHidden(true) end, 3000)
	else
		d(GetString(COMINGBACKHOME_UI_WAITING_FOR_COMBAT_END))
	end
end

--label control for notifications
function CBH.CreateControl()
	TLW = WM:CreateTopLevelWindow("CBH_TLW")
	TLW:SetDimensions(600,120)
	TLW:SetResizeToFitDescendents(true)
	TLW:SetAnchor(CENTER, GuiRoot, CENTER, 0, 110)
	TLW:SetMovable(true)
	TLW:SetMouseEnabled(true)
	TLW:SetClampedToScreen(true)
	TLW:SetTopmost(true)
	TLW:SetHidden(true)

	CreateControl("CBH_Label", TLW, CT_LABEL)
	CBH_Label:SetColor(0.9, 0.9, 0.9, 1)
	CBH_Label:SetFont("ZoFontAlert")
	CBH_Label:SetScale(1)
	CBH_Label:SetMaxLineCount(3)
	CBH_Label:SetWrapMode(TEX_MODE_CLAMP)
	CBH_Label:SetDrawLayer(1)
	CBH_Label:SetAnchor(CENTER, TLW, CENTER, 0, 110)
	CBH_Label:SetDimensions(600,120)
	CBH_Label:SetHorizontalAlignment(1)
end

-- Then we create an event handler function which will be called when the "addon loaded" event
-- occurs. We'll use this to initialize our addon after all of its resources are fully loaded.
function CBH.OnAddOnLoaded(event, addonName)
	-- The event fires each time *any* addon loads - but we only care about when our own addon loads.
	if addonName == CBH.name then
		-- Register our keybinding names
		ZO_CreateStringId("SI_BINDING_NAME_COMING_BACK_HOME", GetString(COMINGBACKHOME_KEYBINDING))

		CBH:Initialize()
		EVENT_MANAGER:UnregisterForEvent(CBH.name, EVENT_ADD_ON_LOADED)
	end
end

-- Finally, we'll register our event handler function to be called when the proper event occurs.
EVENT_MANAGER:RegisterForEvent(CBH.name, EVENT_ADD_ON_LOADED, CBH.OnAddOnLoaded)